#region usings
using System;
using System.ComponentModel.Composition;
using System.Runtime.InteropServices;

using SlimDX;
using SlimDX.Direct3D9;
using System.Collections.Generic;
using VVVV.Core.Logging;
using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.PluginInterfaces.V2.EX9;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
using VVVV.Utils.SlimDX;

#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "DepthMesh", Category = "EX9.Geometry", Help = "Basic template which creates a mesh", Tags = "")]
	#endregion PluginInfo
	public class EX9_GeometryDepthMeshNode : DXMeshOutPluginBase, IPluginEvaluate
	{
		#region fields & pins

		[Input("Depth")]
		IDiffSpread<double> FDepth;

		[Input("Resolution", DefaultValue = 20)]
		IDiffSpread<Vector2D> FResoIn;

		[Import()]
		ILogger FLogger;
		
		List<Vector3> vBuffer = new List<Vector3>();

		#endregion fields & pins

		// import host and hand it to base constructor
		[ImportingConstructor()]
		public EX9_GeometryDepthMeshNode(IPluginHost host) : base(host)
		{
		}

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			//recreate mesh
			if (FResoIn.IsChanged) {
				Reinitialize();
			}

			//update mesh
			if (FDepth.IsChanged) {
				Update();
			}
		}

		//this method gets called, when Reinitialize() was called in evaluate,
		//or a graphics device asks for its data
		protected override Mesh CreateMesh(Device device)
		{
			int faceCount = (int)((FResoIn[0].x-1)*(FResoIn[0].y-1)*2);
			int vertexCount = (int)(FResoIn[0].x*FResoIn[0].y);	
			
			vBuffer.Clear();
			var mesh = new Mesh(device, faceCount , vertexCount, MeshFlags.Managed, VertexFormat.Position);
			var vertices = mesh.LockVertexBuffer(LockFlags.None);
			var indices = mesh.LockIndexBuffer(LockFlags.None);
			
			for (int y=0; y<FResoIn[0].y; y++)
			{
				for (int x=0; x<FResoIn[0].x; x++)
				{
					vBuffer.Add(new Vector3((x/(float)(FResoIn[0].x-1))-0.5f,1-(y/(float)(FResoIn[0].y-1))-0.5f,0.0f));
					vertices.Write(vBuffer[vBuffer.Count-1]);
					if (x>0 && y>0)
					{
						indices.Write((short)(((y-1)*FResoIn[0].x)+x-1));
						indices.Write((short)(((y-1)*FResoIn[0].x)+x));
						indices.Write((short)((y*FResoIn[0].x)+x));
						indices.Write((short)((y*FResoIn[0].x)+x));
						indices.Write((short)((y*FResoIn[0].x)+x-1));
						indices.Write((short)(((y-1)*FResoIn[0].x)+x-1));
					}
				}
			}
			mesh.UnlockVertexBuffer();
			mesh.UnlockIndexBuffer();
			
			return mesh;
		}

		//this method gets called, when Update() was called in evaluate,
		//or a graphics device asks for its mesh, here you can alter the data of the mesh
		protected override void UpdateMesh(Mesh mesh)
		{
			var vertices = mesh.LockVertexBuffer(LockFlags.None);

			for (int i = 0; i < mesh.VertexCount; i++)
				vertices.Write(new Vector3(vBuffer[i].X,vBuffer[i].Y,(float)FDepth[i]));			
			
			mesh.UnlockVertexBuffer();
		}
	}
}
